"""
Extra webassets filters
"""
from webassets.filter import Filter


__all__ = ('RCSSMin',)


class RCSSMin(Filter):
    """Minifies CSS.

    Requires the ``rcssmin`` package (https://github.com/ndparker/rcssmin).
    Alike 'cssmin' it is a port of the YUI CSS compression algorithm but aiming
    for speed instead of maximum compression.
    """

    name = 'rcssmin'

    def setup(self):
        try:
            import rcssmin
        except ImportError:
            raise EnvironmentError('The "rcssmin" package is not installed.')
        else:
            self.rcssmin = rcssmin

    def output(self, _in, out, **kw):
        out.write(self.rcssmin.cssmin(_in.read()))



"""
Main entrance to commandline actions
"""
import click

from boussole.cli.version import version_command
from boussole.cli.compile import compile_command
from boussole.cli.watch import watch_command
from boussole.cli.startproject import startproject_command
from boussole.logs import init_logger


# Help alias on '-h' argument
CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])

# Default logger conf
BOUSSOLE_LOGGER_CONF = ('DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL', None)


@click.group(context_settings=CONTEXT_SETTINGS)
@click.option('-v', '--verbose', type=click.IntRange(min=0, max=5), default=4,
              metavar='INTEGER',
              help="An integer between 0 and 5, where '0' make a totaly "
              "silent output and '5' set level to DEBUG (the most verbose "
              "level). Default to '4' (Info level).")
@click.pass_context
def cli_frontend(ctx, verbose):
    """
    Boussole is a commandline interface to build Sass projects using libsass.

    Every project will need a settings file containing all needed settings to
    build it.
    """
    printout = True
    if verbose == 0:
        verbose = 1
        printout = False

    # Verbosity is the inverse of logging levels
    levels = [item for item in BOUSSOLE_LOGGER_CONF]
    levels.reverse()
    # Init the logger config
    root_logger = init_logger(levels[verbose], printout=printout)

    # Init the default context that will be passed to commands
    ctx.obj = {
        'verbosity': verbose,
        'logger': root_logger,
    }


# Attach commands methods to the main grouper
cli_frontend.add_command(version_command, name="version")
cli_frontend.add_command(compile_command, name="compile")
cli_frontend.add_command(watch_command, name="watch")
cli_frontend.add_command(startproject_command, name="startproject")



# -*- coding: utf-8 -*-
"""
Optimus project for Foundation toast
"""
__version__ = """19.19.0"""
__cookiecutter_context__ = {'_template': '/home/emencia/projects/apps/Pims-dev/cookiecutter-optimus-foundation6', 'directory_name': 'foundation_toast', 'project_name': 'Foundation toast', '_copy_without_render': ['.gitignore', 'boussole.json', 'requirements.txt', 'locale', 'sources']}



# -*- coding: utf-8 -*-
"""
Resolver
========

Resolver is in charge to resolve path in import rules. Resolving is done using
given source directory and libraries directories paths.
"""
import os

from six import string_types

from boussole.exceptions import UnresolvablePath
from boussole.exceptions import UnclearResolution


class ImportPathsResolver(object):
    """
    Import paths resolver.

    Resolve given paths from SCSS source to absolute paths.

    It's a mixin, meaning without own ``__init__`` method so it's should be
    safe enough to inherit it from another class.

    Attributes:
        CANDIDATE_EXTENSIONS (list): List of extensions available to build
            candidate paths. Beware, order does matter, the first extension
            will be the top candidate.
        STRICT_PATH_VALIDATION (bool): A switch to enabled (``True``) or
            disable (``False``) exception raising when a path can not be
            resolved.
    """
    CANDIDATE_EXTENSIONS = ['scss', 'sass', 'css']
    STRICT_PATH_VALIDATION = True

    def candidate_paths(self, filepath):
        """
        Return candidates path for given path

        * If Filename does not starts with ``_``, will build a candidate for
          both with and without ``_`` prefix;
        * Will build For each available extensions if filename does not have
          an explicit extension;
        * Leading path directory is preserved;

        Args:
            filepath (str): Relative path as finded in an import rule from a
                SCSS source.

        Returns:
            list: Builded candidate paths (as relative paths).
        """
        filelead, filetail = os.path.split(filepath)
        name, extension = os.path.splitext(filetail)
        # Removed leading dot from extension
        if extension:
            extension = extension[1:]

        filenames = [name]
        # If underscore prefix is present, dont need to double underscore
        if not name.startswith('_'):
            filenames.append("_{}".format(name))

        # If explicit extension, dont need to add more candidate extensions
        if extension and extension in self.CANDIDATE_EXTENSIONS:
            filenames = [".".join([k, extension]) for k in filenames]
        # Else if no extension or not candidate, add candidate extensions
        else:
            # Restore uncandidate extensions if any
            if extension:
                filenames = [".".join([k, extension]) for k in filenames]
            new = []
            for ext in self.CANDIDATE_EXTENSIONS:
                new.extend([".".join([k, ext]) for k in filenames])
            filenames = new

        # Return candidates with restored leading path if any
        return [os.path.join(filelead, v) for v in filenames]

    def check_candidate_exists(self, basepath, candidates):
        """
        Check that at least one candidate exist into a directory.

        Args:
            basepath (str): Directory path where to search for candidate.
            candidates (list): List of candidate file paths.

        Returns:
            list: List of existing candidates.
        """
        checked = []
        for item in candidates:
            abspath = os.path.join(basepath, item)
            if os.path.exists(abspath):
                checked.append(abspath)

        return checked

    def resolve(self, sourcepath, paths, library_paths=None):
        """
        Resolve given paths from given base paths

        Return resolved path list.

        Note:
            Resolving strategy is made like libsass do, meaning paths in
            import rules are resolved from the source file where the import
            rules have been finded.

            If import rule is not explicit enough and two file are candidates
            for the same rule, it will raises an error. But contrary to
            libsass, this happen also for files from given libraries in
            ``library_paths`` (opposed to libsass just silently taking the
            first candidate).

            https://github.com/sveetch/boussole/issues/37

        Args:
            sourcepath (str): Source file path, its directory is used to
                resolve given paths. The path must be an absolute path to
                avoid errors on resolving.
            paths (list): Relative paths (from ``sourcepath``) to resolve.
            library_paths (list): List of directory paths for libraries to
                resolve paths if resolving fails on the base source path.
                Default to None.

        Raises:
            UnresolvablePath: If a path does not exist and
                ``STRICT_PATH_VALIDATION`` attribute is ``True``.

        Returns:
            list: List of resolved path.
        """
        # Split basedir/filename from sourcepath, so the first resolving
        # basepath is the sourcepath directory, then the optionnal
        # given libraries
        basedir, filename = os.path.split(sourcepath)
        basepaths = [basedir]
        resolved_paths = []

        # Add given library paths to the basepaths for resolving
        # Accept a string if not allready in basepaths
        if library_paths and isinstance(library_paths, string_types) and \
           library_paths not in basepaths:
            basepaths.append(library_paths)
        # Add path item from list if not allready in basepaths
        elif library_paths:
            for k in list(library_paths):
                if k not in basepaths:
                    basepaths.append(k)

        for import_rule in paths:
            candidates = self.candidate_paths(import_rule)

            # Search all existing candidates:
            # * If more than one candidate raise an error;
            # * If only one, accept it;
            # * If no existing candidate raise an error;
            stack = []
            for i, basepath in enumerate(basepaths):
                checked = self.check_candidate_exists(basepath, candidates)
                if checked:
                    stack.extend(checked)

            # More than one existing candidate
            if len(stack) > 1:
                raise UnclearResolution(
                    "rule '{}' This is not clear for these paths: {}".format(
                        import_rule, ', '.join(stack)
                    )
                )
            # Accept the single one
            elif len(stack) == 1:
                resolved_paths.append(os.path.normpath(stack[0]))
            # No validated candidate
            else:
                if self.STRICT_PATH_VALIDATION:
                    raise UnresolvablePath(
                        "Imported path '{}' does not exist in '{}'".format(
                            import_rule, basedir
                        )
                    )

        return resolved_paths



# -*- coding: utf-8 -*-
"""
YAML settings backend
=====================
"""
import yaml
import pyaml

from boussole.exceptions import SettingsBackendError
from boussole.conf.base_backend import SettingsBackendBase


class SettingsBackendYaml(SettingsBackendBase):
    """
    YAML backend for settings

    Use PyYaml for parsing and pyaml for dumping.

    Attributes:
        _default_filename: Filename for settings file to load.
            Value is ``settings.yml``.
        _kind_name: Backend format name.
            Value is ``yaml``.
        _file_extension: Default filename extension.
            Value is ``yml``.
    """
    _default_filename = 'settings.yml'
    _kind_name = 'yaml'
    _file_extension = 'yml'

    def dump(self, content, filepath, indent=4):
        """
        Dump settings content to filepath.

        Args:
            content (str): Settings content.
            filepath (str): Settings file location.
        """
        with open(filepath, 'w') as fp:
            pyaml.dump(content, dst=fp, indent=indent)

    def parse(self, filepath, content):
        """
        Parse opened settings content using YAML parser.

        Args:
            filepath (str): Settings object, depends from backend
            content (str): Settings content from opened file, depends from
                backend.

        Raises:
            boussole.exceptions.SettingsBackendError: If parser can not decode
                a valid YAML object.

        Returns:
            dict: Dictionnary containing parsed setting elements.

        """
        try:
            parsed = yaml.load(content, Loader=yaml.FullLoader)
        except yaml.YAMLError as exc:
            msg = "No YAML object could be decoded from file: {}\n{}"
            raise SettingsBackendError(msg.format(filepath, exc))
        return parsed



# -*- coding: utf-8 -*-
"""
The project pages map for Foundation toast
"""
from optimus.pages.views.base import PageViewBase
from optimus.conf.registry import settings

from .index import Index


# Enabled pages to build
PAGES = [
    Index(destination="index.html"),
    Index(lang="fr_FR"),
]



# -*- coding: utf-8 -*-
"""
Backend discover
================

"""
import os
from collections import OrderedDict

from boussole.exceptions import SettingsDiscoveryError


class Discover:
    """
    Should be able to find a settings file without any specific backend given,
    just a directory path (the base dir) is required.

    So:

    * If a file name is explicitely given, use it to find backend;
    * If no file name is given but a backend is, use its default file name;
    * If file name nor backend is given, start full discover process:

        * Get all backend default settings file name;
        * Search for any of these available settings file names;
        * If no available settings file name if finded, discovering fail;
        * If one file name is given assume backend from file name;

    backends (list): List of backend engines to get available backend engines.
    """

    def __init__(self, backends=[]):
        self.backends = backends

        indexes = self.scan_backends(self.backends)
        self.engines, self.filenames, self.extensions = indexes

    def scan_backends(self, backends):
        """
        From given backends create and return engine, filename and extension
        indexes.

        Arguments:
            backends (list): List of backend engines to scan. Order does matter
                since resulted indexes are stored in an ``OrderedDict``. So
                discovering will stop its job if it meets the first item.

        Returns:
            tuple: Engine, filename and extension indexes where:

            * Engines are indexed on their kind name with their backend object
              as value;
            * Filenames are indexed on their filename with engine kind name as
              value;
            * Extensions are indexed on their extension with engine kind name
              as value;
        """
        engines = OrderedDict()
        filenames = OrderedDict()
        extensions = OrderedDict()

        for item in backends:
            engines[item._kind_name] = item
            filenames[item._default_filename] = item._kind_name
            extensions[item._file_extension] = item._kind_name

        return engines, filenames, extensions

    def get_engine(self, filepath, kind=None):
        """
        From given filepath try to discover which backend format to use.

        Discovering is pretty naive as it find format from file extension.

        Args:
            filepath (str): Settings filepath or filename.

        Keyword Arguments:
            kind (str): A format name to enforce a specific backend. Can be any
                value from attribute ``_kind_name`` of available backend
                engines.

        Raises:
            boussole.exceptions.SettingsDiscoveryError: If extension is
            unknowed or if given format name is unknowed.

        Returns:
            object: Backend engine class.

        """
        if not kind:
            extension = os.path.splitext(filepath)[1]
            if not extension:
                msg = ("Unable to discover settings format from an empty file "
                       "extension: {}")
                raise SettingsDiscoveryError(msg.format(filepath))
            elif extension[1:] not in self.extensions:
                msg = ("Settings file extension is unknowed from available "
                       "backends: {}")
                raise SettingsDiscoveryError(msg.format(filepath))
            kind = self.extensions[extension[1:]]
        elif kind not in self.engines:
            msg = "Given settings format is unknow: {}"
            raise SettingsDiscoveryError(msg.format(kind))

        return self.engines[kind]

    def guess_filename(self, basedir, kind=None):
        """
        Try to find existing settings filename from base directory using
        default filename from available engines.

        First finded filename from available engines win. So registred engines
        order matter.

        Arguments:
            basedir (string): Directory path where to search for.

        Keyword Arguments:
            kind (string): Backend engine kind name to search for default
                settings filename. If not given, search will be made for
                default settings filename from all available backend engines.

        Returns:
            tuple: Absolute filepath and backend engine class.
        """
        if kind:
            filepath = os.path.join(basedir,
                                    self.engines[kind]._default_filename)
            if os.path.exists(filepath):
                return filepath, self.engines[kind]

        for filename, kind in self.filenames.items():
            filepath = os.path.join(basedir, filename)
            if os.path.exists(filepath):
                return filepath, self.engines[kind]

        msg = "Unable to find any settings in directory: {}"
        raise SettingsDiscoveryError(msg.format(basedir))

    def search(self, filepath=None, basedir=None, kind=None):
        """
        Search for a settings file.

        Keyword Arguments:
            filepath (string): Path to a config file, either absolute or
                relative. If absolute set its directory as basedir (omitting
                given basedir argument). If relative join it to basedir.
            basedir (string): Directory path where to search for.
            kind (string): Backend engine kind name (value of attribute
                ``_kind_name``) to help discovering with empty or relative
                filepath. Also if explicit absolute filepath is given, this
                will enforce the backend engine (such as yaml kind will be
                forced for a ``foo.json`` file).

        Returns:
            tuple: Absolute filepath and backend engine class.
        """
        # None values would cause trouble with path joining
        if filepath is None:
            filepath = ''
        if basedir is None:
            basedir = '.'

        if not basedir and not filepath:
            msg = "Either basedir or filepath is required for discovering"
            raise SettingsDiscoveryError(msg)

        if kind and kind not in self.engines:
            msg = "Given settings format is unknow: {}"
            raise SettingsDiscoveryError(msg.format(kind))

        # Implicit filename to find from backend
        if not filepath:
            filename, engine = self.guess_filename(basedir, kind)
            filepath = os.path.join(basedir, filename)
        # Explicit filename dont have to search for default backend file and
        # blindly force given backend if any
        else:
            if os.path.isabs(filepath):
                basedir, filename = os.path.split(filepath)
            else:
                filepath = os.path.join(basedir, filepath)

            if not os.path.exists(filepath):
                msg = "Given settings file does not exists: {}"
                raise SettingsDiscoveryError(msg.format(filepath))

            engine = self.get_engine(filepath, kind)

        return filepath, engine



# -*- coding: utf-8 -*-
from optimus.pages.views.base import PageViewBase
from optimus.conf.registry import settings


class Index(PageViewBase):
    """
    Default index page
    """
    title = "My project"
    template_name = "index.html"
    destination = "{language_code}/index.html"

    def get_context(self):
        super().get_context()

        self.context.update({
            'DEFAULT_LANGUAGE': settings.LANGUAGE_CODE,
            'LANGUAGES': settings.LANGUAGES,
        })

        return self.context



# -*- coding: utf-8 -*-
import os
import click
import logging

import six

from boussole.exceptions import SettingsInvalidError
from boussole.project import ProjectStarter


@click.command('startproject', short_help="Create a new Sass project.")
@click.option('--basedir', metavar='PATH',
              prompt="Project base directory",
              help=("Base directory where settings filename and project "
                    "structure will be created."),
              default=".")
@click.option('--sourcedir', metavar='PATH',
              prompt="Sources directory",
              help="Directory (within base dir) for your Sass sources.",
              default="scss")
@click.option('--targetdir', metavar='PATH',
              prompt="Target directory",
              help="Directory (within base dir) where to write compiled "
                   "files.",
              default="css")
@click.option('--backend', metavar='STRING',
              prompt="Settings format name",
              type=click.Choice(['json', 'yaml']),
              help="Settings format name",
              default="json")
@click.option('--config', metavar='PATH',
              help="Settings file name",
              default=None)
@click.pass_context
def startproject_command(context, basedir, sourcedir, targetdir,
                         backend, config):
    """
    Create a new Sass project

    This will prompt you to define your project configuration in a settings
    file then create needed directory structure.

    Arguments 'basedir', 'config', 'sourcedir', 'targetdir' can not be empty.

    'backend' argument is optionnal, its value can be "json" or "yaml" and its
    default value is "json".

    If "backend" is given and 'config' is empty, this will change the default
    value for 'config' argument such as with "json" filename will be
    "settings.json" and for "yaml" it will be "settings.yml".
    """
    logger = logging.getLogger("boussole")

    starter = ProjectStarter(backend_name=backend)

    # If not given, config file name is setted from backend default filename
    if not config:
        config = starter.backend_engine._default_filename

    try:
        results = starter.init(*(
            basedir,
            config,
            sourcedir,
            targetdir,
        ), cwd=os.getcwd())
    except SettingsInvalidError as e:
        logger.critical(six.text_type(e))
        raise click.Abort()
    else:
        logger.info(u"Project directory structure and configuration file have "
                    "been created.")

        logger.info(u"Now you should start to create some Sass sources into "
                    "'{}', then compile them "
                    "using:".format(results['sourcedir']))

        logger.info(u"    boussole compile "
                    "--config={}".format(results['config']))



# -*- coding: utf-8 -*-
import time
import os
import click
import logging

import six

from watchdog.observers import Observer
from watchdog.observers.polling import PollingObserver

from boussole.conf.discovery import Discover
from boussole.conf.json_backend import SettingsBackendJson
from boussole.conf.yaml_backend import SettingsBackendYaml
from boussole.exceptions import BoussoleBaseException
from boussole.inspector import ScssInspector
from boussole.project import ProjectBase
from boussole.watcher import (WatchdogLibraryEventHandler,
                              WatchdogProjectEventHandler)


@click.command("watch", short_help="Watch for change on your Sass project.")
@click.option("--backend", metavar="STRING",
              type=click.Choice(["json", "yaml"]),
              help="Settings format name",
              default="json")
@click.option("--config", default=None, metavar="PATH",
              help="Path to a Boussole config file",
              type=click.Path(exists=True))
@click.option("--poll", is_flag=True, help="Use Watchdog polling observer")
@click.pass_context
def watch_command(context, backend, config, poll):
    """
    Watch for change on your Sass project sources then compile them to CSS.

    Watched events are:

    \b
    * Create: when a new source file is created;
    * Change: when a source is changed;
    * Delete: when a source is deleted;
    * Move: When a source file is moved in watched dirs. Also occurs with
      editor transition file;

    Almost all errors occurring during compile won"t break watcher, so you can
    resolve them and watcher will try again to compile once a new event
    occurs.

    You can stop watcher using key combo "CTRL+C" (or CMD+C on MacOSX).
    """
    logger = logging.getLogger("boussole")
    logger.info("Watching project")

    # Discover settings file
    try:
        discovering = Discover(backends=[SettingsBackendJson,
                                         SettingsBackendYaml])
        config_filepath, config_engine = discovering.search(
            filepath=config,
            basedir=os.getcwd(),
            kind=backend
        )

        project = ProjectBase(backend_name=config_engine._kind_name)
        settings = project.backend_engine.load(filepath=config_filepath)
    except BoussoleBaseException as e:
        logger.critical(six.text_type(e))
        raise click.Abort()

    logger.debug(u"Settings file: {} ({})".format(
                 config_filepath, config_engine._kind_name))
    logger.debug(u"Project sources directory: {}".format(
                settings.SOURCES_PATH))
    logger.debug(u"Project destination directory: {}".format(
                settings.TARGET_PATH))
    logger.debug(u"Exclude patterns: {}".format(
                settings.EXCLUDES))

    # Watcher settings
    watcher_templates_patterns = {
        "patterns": ["*.scss", "*.sass"],
        "ignore_patterns": ["*.part"],
        "ignore_directories": False,
        "case_sensitive": True,
    }

    # Init inspector instance shared through all handlers
    inspector = ScssInspector()

    if not poll:
        logger.debug(u"Using Watchdog native platform observer")
        observer = Observer()
    else:
        logger.debug(u"Using Watchdog polling observer")
        observer = PollingObserver()

    # Init event handlers
    project_handler = WatchdogProjectEventHandler(settings, inspector,
                                                  **watcher_templates_patterns)

    lib_handler = WatchdogLibraryEventHandler(settings, inspector,
                                              **watcher_templates_patterns)

    # Observe source directory
    observer.schedule(project_handler, settings.SOURCES_PATH, recursive=True)

    # Also observe libraries directories
    for libpath in settings.LIBRARY_PATHS:
        observer.schedule(lib_handler, libpath, recursive=True)

    # Start watching
    logger.warning(u"Launching the watcher, use CTRL+C to stop it")
    observer.start()

    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        logger.warning(u"CTRL+C used, stopping..")
        observer.stop()

    observer.join()



# -*- coding: utf-8 -*-
"""
Project configuration
=====================

Boussole work on per project configurations stored in a settings file for each
project.

Backends behavior is to search for a settings file in the given directory, read
it, possibly patch its values and then return a
:class:`boussole.conf.model.Settings object`.

Almost all paths in settings will be expanded to absolute paths if they are
not allready so:

* If the path start with a home directory character, the home directory is used
  to expand the path;
* If the path is relative, expand it to absolute using directory from settings
  file location;

Also note, that Sass files from libraries directories are never compiled.
"""
# Manifest define default values and post process methods for each setting
# Warning: Order does matter on "postprocess" methods
SETTINGS_MANIFEST = {
    'LIBRARY_PATHS': {
        'default': [],
        'postprocess': (
            '_patch_expand_paths',
            '_validate_paths',
        ),
    },
    'SOURCES_PATH': {
        'default': None,
        'postprocess': (
            '_validate_required',
            '_patch_expand_path',
            '_validate_path',
        ),
    },
    'TARGET_PATH': {
        'default': None,
        'postprocess': (
            '_validate_required',
            '_patch_expand_path',
            '_validate_path',
        ),
    },
    'OUTPUT_STYLES': {
        'default': 'nested',
        'postprocess': [],
    },
    'SOURCE_COMMENTS': {
        'default': False,
        'postprocess': [],
    },
    'SOURCE_MAP': {
        'default': False,
        'postprocess': [],
    },
    'EXCLUDES': {
        'default': [],
        'postprocess': [],
    },
}



# -*- coding: utf-8 -*-
"""
Base settings for Foundation toast
"""
import os
from webassets import Bundle

# Register custom webasset filter for RCssMin minifier
from webassets.filter import register_filter
from .webassets_filters import RCSSMin
register_filter(RCSSMin)

DEBUG = True

PROJECT_DIR = os.path.abspath(
    os.path.join(
        os.path.dirname(__file__),
        "..",
    )
)

# Common site name and domain to use available in templates
SITE_NAME = 'Foundation toast'
SITE_DOMAIN = 'localhost:8001'

# Sources directory where the assets will be searched
SOURCES_DIR = os.path.join(PROJECT_DIR, 'sources')
# Templates directory
TEMPLATES_DIR = os.path.join(SOURCES_DIR, 'templates')
# Directory where all stuff will be builded
PUBLISH_DIR = os.path.join(PROJECT_DIR, '_build/dev')
# Path where will be moved all the static files, usually this is a directory in
# the ``PUBLISH_DIR``
STATIC_DIR = os.path.join(PROJECT_DIR, PUBLISH_DIR, 'static')
# Path to the i18n messages catalog directory
LOCALES_DIR = os.path.join(PROJECT_DIR, 'locale')

# Locale name for default language to use for Pages
LANGUAGE_CODE = "en_US"

# A list of locale name for all available languages to manage with PO files
LANGUAGES = (LANGUAGE_CODE, "fr_FR")

# The static url to use in templates and with webassets
# This can be a full URL like http://, a relative path or an absolute path
STATIC_URL = 'static/'

# Extra or custom bundles
BUNDLES = {
    'main_css': Bundle(
        'css/main.css',
        filters='rcssmin',
        output='css/main.min.css'
    ),
    'main_js': Bundle(
        "js/foundation/vendor.js",
        "js/foundation/plugins/foundation.core.js",
        "js/foundation/plugins/foundation.util.box.js",
        "js/foundation/plugins/foundation.util.imageLoader.js",
        "js/foundation/plugins/foundation.util.keyboard.js",
        "js/foundation/plugins/foundation.util.mediaQuery.js",
        "js/foundation/plugins/foundation.util.motion.js",
        "js/foundation/plugins/foundation.util.nest.js",
        "js/foundation/plugins/foundation.util.timer.js",
        "js/foundation/plugins/foundation.util.touch.js",
        "js/foundation/plugins/foundation.util.triggers.js",
        "js/foundation/plugins/foundation.abide.js",
        "js/foundation/plugins/foundation.accordion.js",
        "js/foundation/plugins/foundation.accordionMenu.js",
        "js/foundation/plugins/foundation.drilldown.js",
        "js/foundation/plugins/foundation.dropdown.js",
        "js/foundation/plugins/foundation.dropdownMenu.js",
        "js/foundation/plugins/foundation.equalizer.js",
        "js/foundation/plugins/foundation.interchange.js",
        "js/foundation/plugins/foundation.smoothScroll.js",
        "js/foundation/plugins/foundation.magellan.js",
        "js/foundation/plugins/foundation.offcanvas.js",
        "js/foundation/plugins/foundation.orbit.js",
        "js/foundation/plugins/foundation.responsiveMenu.js",
        "js/foundation/plugins/foundation.responsiveToggle.js",
        "js/foundation/plugins/foundation.reveal.js",
        "js/foundation/plugins/foundation.slider.js",
        "js/foundation/plugins/foundation.sticky.js",
        "js/foundation/plugins/foundation.tabs.js",
        "js/foundation/plugins/foundation.toggler.js",
        "js/foundation/plugins/foundation.tooltip.js",
        "js/foundation/plugins/foundation.responsiveAccordionTabs.js",
        "js/main.js",
        filters='rjsmin',
        output='js/main.min.js'
    ),
}

# Sources files or directory to synchronize within the static directory
FILES_TO_SYNC = (
    #'images',
    #'fonts',
    'css',
)



# -*- coding: utf-8 -*-
"""
Base settings backend
=====================

Backends are responsible to find settings file, parse it, load its values then
return a Settings object.

Backends inherit from :class:`boussole.conf.post_processor` so they can post
process each loaded settings values following the settings manifest rules.

Actually available backends are JSON and YAML.

"""
import io
import os

from boussole.exceptions import SettingsBackendError
from boussole.conf.model import Settings
from boussole.conf.post_processor import SettingsPostProcessor


class SettingsBackendBase(SettingsPostProcessor):
    """
    Base project settings backend

    Args:
        basedir (str): Directory path where to search for settings filepath.

            Default is empty, meaning it will resolve path from current
            directory. Don't use an empty ``basedir`` attribute to load
            settings from non-absolute filepath.

            Given value will fill intial value for ``projectdir`` attribute.

    Attributes:
        _default_filename: Filename for settings file to load.
            Value is ``settings.txt``.
        _kind_name: Backend format name.
            Value is ``txt``.
        _file_extension: Default filename extension.
            Value is ``txt``.
    """
    _default_filename = 'settings.txt'
    _kind_name = 'txt'
    _file_extension = 'txt'

    def __init__(self, basedir=None):
        self.basedir = basedir or ''
        self.projectdir = self.basedir

    def parse_filepath(self, filepath=None):
        """
        Parse given filepath to split possible path directory from filename.

        * If path directory is empty, will use ``basedir`` attribute as base
          filepath;
        * If path directory is absolute, ignore ``basedir`` attribute;
        * If path directory is relative, join it to ``basedir`` attribute;

        Keyword Arguments:
            filepath (str): Filepath to use to search for settings file. Will
                use value from ``_default_filename`` class attribute if empty.

                If filepath contain a directory path, it will be splitted from
                filename and used as base directory (and update object
                ``basedir`` attribute).

        Returns:
            tuple: Separated path directory and filename.
        """
        filepath = filepath or self._default_filename

        path, filename = os.path.split(filepath)

        if not path:
            path = self.basedir
        elif not os.path.isabs(path):
            path = os.path.join(self.basedir, path)

        return os.path.normpath(path), filename

    def check_filepath(self, path, filename):
        """
        Check and return the final filepath to settings

        Args:
            path (str): Directory path where to search for settings file.
            filename (str): Filename to use to search for settings file.

        Raises:
            boussole.exceptions.SettingsBackendError: If determined filepath
                does not exists or is a directory.

        Returns:
            string: Settings file path, joining given path and filename.

        """
        settings_path = os.path.join(path, filename)

        if not os.path.exists(settings_path) or \
           not os.path.isfile(settings_path):
            msg = "Unable to find settings file: {}"
            raise SettingsBackendError(msg.format(settings_path))

        return settings_path

    def open(self, filepath):
        """
        Open settings backend to return its content

        Args:
            filepath (str): Settings object, depends from backend

        Returns:
            string: File content.

        """
        with io.open(filepath, 'r', encoding='utf-8') as fp:
            content = fp.read()
        return content

    def parse(self, filepath, content):
        """
        Load and parse opened settings content.

        Base method do nothing because parsing is dependent from backend.

        Args:
            filepath (str): Settings file location.
            content (str): Settings content from opened file, depends from
                backend.

        Returns:
            dict: Dictionnary containing parsed setting options.

        """
        return {}

    def dump(self, content, filepath):
        """
        Dump settings content to filepath.

        Base method do nothing because dumping is dependent from backend.

        Args:
            content (str): Settings content.
            filepath (str): Settings file location.

        Returns:
            dict: Dictionnary containing parsed setting options.

        """
        return {}

    def clean(self, settings):
        """
        Clean given settings for backend needs.

        Default backend only apply available post processor methods.

        Args:
            dict: Loaded settings.

        Returns:
            dict: Settings object cleaned.

        """
        return self.post_process(settings)

    def load(self, filepath=None):
        """
        Load settings file from given path and optionnal filepath.

        During path resolving, the ``projectdir`` is updated to the file path
        directory.

        Keyword Arguments:
            filepath (str): Filepath to the settings file.

        Returns:
            boussole.conf.model.Settings: Settings object with loaded options.

        """
        self.projectdir, filename = self.parse_filepath(filepath)

        settings_path = self.check_filepath(self.projectdir, filename)

        parsed = self.parse(settings_path, self.open(settings_path))

        settings = self.clean(parsed)

        return Settings(initial=settings)
